# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from amqcfg.query import filter_template_list

basic_template_list = [
    'a.xml',
    'a.txt',
    'b.xml',
    'b.txt',
    'c.xml',
    'c.properties',
]

dataset_filter_basic = (
    ([''], basic_template_list),
    (['a.*'], ['a.xml', 'a.txt']),
    (['b.*'], ['b.xml', 'b.txt']),
    (['.*xml'], ['a.xml', 'b.xml', 'c.xml']),
    (['.*xml', '.*txt'], ['a.xml', 'a.txt', 'b.xml', 'b.txt', 'c.xml']),
    (['xxx'], []),
)


@pytest.mark.parametrize('output_filter,expected_result', dataset_filter_basic)
def test_filter_basic(output_filter, expected_result):
    result = filter_template_list(basic_template_list,
                                  output_filter=output_filter)
    assert result == expected_result



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock
import pytest

import amqcfg.files
from amqcfg.exceptions import ProfileError
from .fakes import fake_module_path, fake_profile_path, fake_os_abspath


@mock.patch('amqcfg.files.get_module_path', side_effect=fake_module_path)
@mock.patch('os.path.isfile', side_effect=(False, True))
def test_packaged_true(*_):
    """Packaged profile selection"""
    expected_name = 'packaged_profile.yaml'
    expected_path = fake_profile_path()
    result_name, result_path = amqcfg.files.select_profile_file(expected_name)
    assert result_name == expected_name
    assert result_path == expected_path


@mock.patch('amqcfg.files.get_module_path', side_effect=fake_module_path)
@mock.patch('os.path.isfile', side_effect=(True, True))
@mock.patch('os.path.abspath', side_effect=fake_os_abspath)
def test_user_true(*_):
    """User Specified profile selection"""
    expected_name = 'user_profile.yaml'
    expected_path = os.path.dirname(fake_os_abspath(''))
    result_name, result_path = amqcfg.files.select_profile_file(expected_name)
    assert result_name == expected_name
    assert result_path == expected_path


@mock.patch('amqcfg.files.get_module_path', side_effect=fake_module_path)
@mock.patch('os.path.isfile', side_effect=(False, False))
def test_non_existing(*_):
    """Non-existing packaged profile selection"""
    expected_name = 'non_existing_packaged_profile.yaml'
    with pytest.raises(ProfileError):
        amqcfg.files.select_profile_file(expected_name)


@mock.patch('amqcfg.files.get_module_path', side_effect=fake_module_path)
@mock.patch('os.path.isfile', side_effect=(False, False))
@mock.patch('os.path.abspath', side_effect=fake_os_abspath)
def test_user_non_existing(*_):
    """Non-existing user profile selection"""
    expected_name = 'non_existing_user_profile.yaml'
    with pytest.raises(ProfileError):
        amqcfg.files.select_profile_file(expected_name)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

import amqcfg.files
from .fakes import fake_module_path, fake_profile_path, \
    fake_templates_path


@mock.patch('amqcfg.files.get_module_path', side_effect=fake_module_path)
def test_get_profiles_path(_):
    expected = fake_profile_path()
    assert amqcfg.files.get_profiles_path() == expected


@mock.patch('amqcfg.files.get_module_path', side_effect=fake_module_path)
def test_get_templates_path(_):
    expected = fake_templates_path()
    assert amqcfg.files.get_templates_path() == expected



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import jinja2
from jinja2 import Environment, FileSystemLoader

from .exceptions import TemplateError
from .files import get_templates_path, select_template_dir

LOG = logging.getLogger(__name__)


def get_template_environment(template_name):
    """Create jinja2 environment for selected template

    :param template_name: name of template set
        (alternatively path to user specified template set)
    :type template_name: str

    :return: jinja2 environment
    :rtype: Environment
    """
    templates_path = get_templates_path()

    selected_template_path = select_template_dir(template_name)

    try:
        env = Environment(
            loader=FileSystemLoader([
                selected_template_path,  # selected template
                templates_path,  # all templates for includes
            ]),
            trim_blocks=True,
            lstrip_blocks=True,
        )
    except jinja2.TemplateError:
        LOG.exception('Original Error')
        # this is amqcfg.exceptions.TemplateError
        raise TemplateError(
            'There was a problem with templating environment.'
        )

    return env



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import pytest
import yaml

import amqcfg.profiles
from amqcfg.exceptions import ProfileError
from amqcfg.profiles import get_tuned_profile


@mock.patch('amqcfg.profiles.load_profile_defaults',
            side_effect=({'var': 'value'},))
@mock.patch('amqcfg.profiles.open', mock.MagicMock())
@mock.patch('yaml.load', mock.Mock())
@mock.patch('amqcfg.profiles.get_profile_template', mock.Mock())
def test_no_tuning(*_):
    profile_name = 'profile.yaml'
    tuning_files = None
    expected_data = 'key: value\n'

    # simulating jinja profile rendering
    fake_profile = mock.Mock()
    amqcfg.profiles.get_profile_template.return_value = fake_profile
    fake_profile.render.return_value = expected_data

    profile_data = get_tuned_profile(profile_name, tuning_files)

    assert profile_data == expected_data
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.load_profile_defaults.assert_called_with(profile_name)
    # noinspection PyUnresolvedReferences
    yaml.load.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.open.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.get_profile_template.assert_called_with(profile_name)
    fake_profile.render.assert_called_with(
        {'var': 'value'}
    )


@mock.patch('amqcfg.profiles.load_profile_defaults',
            side_effect=({'var': 'value'},))
@mock.patch('amqcfg.profiles.open', mock.MagicMock())
@mock.patch('yaml.load', side_effect=({'a': 1}, {'b': 2},))
@mock.patch('amqcfg.profiles.get_profile_template', mock.Mock())
def test_tuning(*_):
    profile_name = 'profile.yaml'
    tuning_files = ['a', 'b']
    expected_data = 'key: value\n'

    # simulating jinja profile rendering
    fake_profile = mock.Mock()
    amqcfg.profiles.get_profile_template.return_value = fake_profile
    fake_profile.render.return_value = expected_data

    profile_data = get_tuned_profile(profile_name, tuning_files)

    assert profile_data == expected_data
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.load_profile_defaults.assert_called_with(profile_name)
    # noinspection PyUnresolvedReferences
    yaml.load.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.open.assert_has_calls([
        mock.call('a', 'r'), mock.call('b', 'r')
    ])
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.get_profile_template.assert_called_with(profile_name)
    fake_profile.render.assert_called_with(
        {'var': 'value', 'a': 1, 'b': 2}
    )


@mock.patch('amqcfg.profiles.load_profile_defaults',
            side_effect=({'var': 'value'},))
@mock.patch('amqcfg.profiles.open',
            side_effect=IOError)
@mock.patch('yaml.load', mock.Mock())
@mock.patch('amqcfg.profiles.get_profile_template', mock.Mock())
def test_bad_file_exception(*_):
    profile_name = 'profile.yaml'
    tuning_files = ['bad']
    expected_data = 'key: value\n'

    # simulating jinja profile rendering
    fake_profile = mock.Mock()
    amqcfg.profiles.get_profile_template.return_value = fake_profile
    fake_profile.render.return_value = expected_data

    with pytest.raises(ProfileError):
        get_tuned_profile(profile_name, tuning_files)

    # noinspection PyUnresolvedReferences
    amqcfg.profiles.load_profile_defaults.assert_called_with(profile_name)
    # noinspection PyUnresolvedReferences
    yaml.load.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.open.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.get_profile_template.assert_not_called()
    fake_profile.render.assert_not_called()


@mock.patch('amqcfg.profiles.load_profile_defaults',
            side_effect=ProfileError)
@mock.patch('amqcfg.profiles.open',
            side_effect=('%this is not yaml',))
@mock.patch('yaml.load', mock.Mock())
@mock.patch('amqcfg.profiles.get_profile_template', mock.Mock())
def test_bad_profile_exception(*_):
    profile_name = 'profile.yaml'
    tuning_files = ['bad']
    expected_data = 'key: value\n'

    # simulating jinja profile rendering
    fake_profile = mock.Mock()
    amqcfg.profiles.get_profile_template.return_value = fake_profile
    fake_profile.render.return_value = expected_data

    with pytest.raises(ProfileError):
        get_tuned_profile(profile_name, tuning_files)

    # noinspection PyUnresolvedReferences
    amqcfg.profiles.load_profile_defaults.assert_called_with(profile_name)
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.open.assert_not_called()
    # noinspection PyUnresolvedReferences
    yaml.load.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.get_profile_template.assert_not_called()
    fake_profile.render.assert_not_called()


@mock.patch('amqcfg.profiles.load_profile_defaults',
            side_effect=({'var': 'value'},))
@mock.patch('amqcfg.profiles.open',
            side_effect=('%this is not yaml',))
# @mock.patch('yaml.load', mock.Mock())
@mock.patch('amqcfg.profiles.get_profile_template', mock.Mock())
def test_bad_yaml_data(*_):
    profile_name = 'profile.yaml'
    tuning_files = ['bad']
    expected_data = 'key: value\n'

    # simulating jinja profile rendering
    fake_profile = mock.Mock()
    amqcfg.profiles.get_profile_template.return_value = fake_profile
    fake_profile.render.return_value = expected_data

    with pytest.raises(ProfileError):
        get_tuned_profile(profile_name, tuning_files)

    # noinspection PyUnresolvedReferences
    amqcfg.profiles.load_profile_defaults.assert_called_with(profile_name)
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.open.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.get_profile_template.assert_not_called()
    fake_profile.render.assert_not_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import os.path
import pytest
import mock

from amqcfg.files import ensure_output_path


@mock.patch('os.path.isdir', side_effect=(False,))
@mock.patch('os.path.isfile', side_effect=(False,))
def test_create_true(*_):
    output_path = '/location/group/output/'
    with mock.patch('os.mkdir', mock.Mock()):
        ensure_output_path(output_path=output_path)
        # noinspection PyUnresolvedReferences
        os.mkdir.assert_called()


@mock.patch('os.path.isdir', side_effect=(True,))
def test_exists(*_):
    output_path = '/location/group/output/'
    with mock.patch('os.mkdir', mock.Mock()):
        ensure_output_path(output_path=output_path)
        # noinspection PyUnresolvedReferences
        os.mkdir.assert_not_called()


@mock.patch('os.path.isdir', side_effect=(False,))
@mock.patch('os.path.isfile', side_effect=(True,))
def test_exists_but_file_exception(*_):
    output_path = '/location/group/output/'
    with mock.patch('os.mkdir', mock.Mock()):
        with pytest.raises(IOError):
            ensure_output_path(output_path=output_path)
        # noinspection PyUnresolvedReferences
        os.mkdir.assert_not_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

import amqcfg.config_data

dataset_metadata_members = (
    'tool_name',
    'datetime',
)


@pytest.mark.parametrize('member', dataset_metadata_members)
def test_add_template_metadata_check_member(member):
    data = {}
    amqcfg.config_data.add_template_metadata(data)
    assert member in data['metadata']


dataset_metadata_datetime_members = (
    'datetime',
    'year',
    'time',
    'date',
    'unix',
)


@pytest.mark.parametrize('member', dataset_metadata_datetime_members)
def test_add_template_metadata_datetime_check_member(member):
    data = {}
    amqcfg.config_data.add_template_metadata(data)
    assert member in data['metadata']['datetime']



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from amqcfg.files import get_output_filename


def test_jinja_suffix():
    assert get_output_filename('my_config.xml.jinja2') == 'my_config.xml'


def test_base_name():
    assert get_output_filename('my_config.xml') == 'my_config.xml'


dataset_incorrect_values = (
    None,
    1,
    1.0,
    [],
    {},
)


@pytest.mark.parametrize('value', dataset_incorrect_values)
def test_int(value):
    with pytest.raises(TypeError):
        get_output_filename(value)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock

from amqcfg.query import list_templates
from ..files.fakes import fake_module_path, fake_templates_path


def fake_walk_templates_basic(*_):
    prefix_path = fake_templates_path()
    return [
        (
            os.path.join(prefix_path, 'a'),
            ['modules'],
            ['broker.xml.jinja2', '_template']
        ),
        (
            os.path.join(prefix_path, 'a/modules'),
            ['broker_xml'],
            ['add-on.jinja2']
        ),
        (
            os.path.join(prefix_path, 'a/modules/broker_xml'),
            [],
            ['queues.jinja2']
        ),
        (
            os.path.join(prefix_path, 'b'),
            ['1.0.0'], []
        ),
        (
            os.path.join(prefix_path, 'b/1.0.0'),
            ['modules'],
            ['management.xml.jinja2', 'artemis-roles.properties.jinja2',
             'broker.xml.jinja2', 'artemis.profile.jinja2',
             'bootstrap.xml.jinja2', 'jolokia-access.xml.jinja2',
             'artemis-users.properties.jinja2',
             'logging.properties.jinja2',
             'login.config.jinja2', '_template']
        ),
        (
            os.path.join(prefix_path, 'b/1.0.0/modules'),
            [], ['broker_xml.jinja2']
        ),
        (
            os.path.join(prefix_path, 'not_a_template'),
            ['a'],
            ['acceptors.jinja2', 'store.jinja2',
             'cluster_connections.jinja2',
             'security_settings.jinja2', 'journal.jinja2',
             'broadcast_group.jinja2', 'discovery_group.jinja2',
             'cluster.jinja2', 'addresses.jinja2',
             'critical_analyzer.jinja2',
             'grouping_handler.jinja2', 'ha_policy.jinja2',
             'duplicate_cache.jinja2', 'queue.jinja2',
             'broker_plugins.jinja2',
             'wildcards_addresses.jinja2', 'address_settings.jinja2',
             'connectors.jinja2']),
        (
            os.path.join(prefix_path, 'not_a_template/a'),
            ['x'],
            ['security_setting.jinja2', 'security_setting_plugin.jinja2']
        ),
        (
            os.path.join(prefix_path, 'not_a_template/a/x'),
            [],
            ['jdbc.jinja2']
        ),
    ]


@mock.patch('amqcfg.query.get_module_path', side_effect=fake_module_path)
@mock.patch('os.walk', side_effect=fake_walk_templates_basic)
def test_true(*_):
    expected = [
        'a',
        'b/1.0.0',
    ]

    result = list_templates()
    assert expected == result


def fake_walk_templates_not_a_template(*_):
    prefix_path = fake_templates_path()
    return [
        (
            os.path.join(prefix_path, 'not_a_template'),
            ['a'],
            ['acceptors.jinja2', 'store.jinja2',
             'cluster_connections.jinja2',
             'security_settings.jinja2', 'journal.jinja2',
             'broadcast_group.jinja2', 'discovery_group.jinja2',
             'cluster.jinja2', 'addresses.jinja2',
             'critical_analyzer.jinja2',
             'grouping_handler.jinja2', 'ha_policy.jinja2',
             'duplicate_cache.jinja2', 'queue.jinja2',
             'broker_plugins.jinja2',
             'wildcards_addresses.jinja2', 'address_settings.jinja2',
             'connectors.jinja2']),
        (
            os.path.join(prefix_path, 'not_a_template/a'),
            ['x'],
            ['security_setting.jinja2', 'security_setting_plugin.jinja2']
        ),
        (
            os.path.join(prefix_path, 'not_a_template/a/x'),
            [],
            ['jdbc.jinja2']
        ),
    ]


@mock.patch('amqcfg.query.get_module_path', side_effect=fake_module_path)
@mock.patch('os.walk', side_effect=fake_walk_templates_not_a_template)
def test_not_a_template(*_):
    expected = []

    result = list_templates()
    assert expected == result


@mock.patch('amqcfg.query.get_module_path', side_effect=fake_module_path)
@mock.patch('os.walk', side_effect=((),))
def test_empty(*_):
    expected = []

    result = list_templates()
    assert expected == result



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import copy
import json
import logging

import jinja2
from jinja2 import Environment, Template

from .config_data import add_template_metadata, add_render_config
from .exceptions import TemplateError, GenerationError
from .files import ensure_output_path, get_output_filename
from .output import write_output
from .profiles import load_tuned_profile
from .query import filter_template_list, get_main_template_list
from .templates import get_template_environment

# workaround for flake8: F401 'jinja2.Template' imported but unused
_ = Template
_ = Environment

LOG = logging.getLogger(__name__)


def generate(profile, template=None, output_path=None, output_filter=None,
             render_options=None, tuning_files=None, write_profile_data=False):
    """Generate procedure, do the main generation procedure

    generate output files based on output_filter, from selected template set,
    with selected profile, and write output to a proposed output path.
    :param profile: name of packaged profile,
        or path to user provided profile
    :type profile: str
    :param template: name of packaged template set,
        or path to user provided template set
    :type template: str or None
    :param output_path: proposed output path,
        if do not exists, it will be created
    :type output_path: str or None
    :param output_filter: list of regular expressions to filter out which
        output files should be generated, if None, then all will be generated,
        based on selected template set
    :type output_filter: list[str] or None
    :param render_options: extra render options tuning
    :type render_options: RenderOptions
    :param tuning_files: Additional user values to fine-tune profile before
        applying it to template.
    :type tuning_files: list[str] or None
    :param write_profile_data: enables writing profile data used for templating
        to file in a output path, output path must be specified
    :type write_profile_data: bool

    :return: mapping of filename to generated data for further use
    :rtype: dict[str, str] or dict[str, unicode]
    """
    config_data, tuned_profile = load_tuned_profile(profile, tuning_files)

    add_template_metadata(config_data)
    if render_options:
        add_render_config(config_data, render_options)

    # template
    if template is None:
        template = config_data.get('render', {}).get('template')
        LOG.debug('Profile specified template: %s', template)
    if template is None:
        raise TemplateError(
            'Missing template. User nor profile specifies a template.'
        )

    env = get_template_environment(template)

    template_list = get_main_template_list(env)
    if output_filter:
        template_list = filter_template_list(template_list, output_filter)

    LOG.debug('Config data: %s', json.dumps(config_data))

    if output_path:
        ensure_output_path(output_path)
        if write_profile_data:
            write_output('profile_data.yaml', output_path, tuned_profile)

    return generate_outputs(config_data, template_list, env, output_path)


# main alias
main = generate


def generate_outputs(config_data, template_list, env, output_path=None):
    """Generate output files based on config_data, (filtered) template list,
    within provided jinja environment, and if output_path is specified, then
    write results to that directory.

    .. note: output_path directory has to be created before.

    :param config_data: configuration data mapping for templating
    :type config_data: dict
    :param template_list: list of main template file names
    :type template_list: list[str]
    :param env: jinja2 template environment
    :type env: Environment
    :param output_path: path where to generate output files,
        or None to do a dry run
    :type output_path: str

    :raises GenerationError: when there was a problem with generating one of
        config files

    :return: mapping of filename to generated data for further use
    :rtype: dict[str, str] or dict[str, unicode]
    """
    generate_exception = None
    result_data = {}

    # only additions of runtime data happens on base level,
    # so deeper structures are unaffected
    config_data = copy.copy(config_data)

    # metadata structure initialization,
    # if called without add_template_metadata()
    if 'metadata' not in config_data:
        config_data['metadata'] = {}

    for template_name in template_list:
        out_filename = get_output_filename(template_name)
        config_data['metadata']['out_filename'] = template_name

        try:
            template = env.get_template(template_name)
            output_data = template.render(config_data)
        except jinja2.TemplateError as exc:
            LOG.error('Config file "%s" generation FAILED', out_filename)
            LOG.exception('Original error')
            if not generate_exception:
                generate_exception = GenerationError(
                    'There was a problem generating file "%s" with "%s" '
                    'template: %s'
                    % (out_filename, template_name, exc)
                )
        else:
            LOG.debug('BEGIN %s\n%s', out_filename, output_data)
            LOG.debug('END %s' % out_filename)
            LOG.info('Config file "%s" generation PASSED', out_filename)

            result_data[out_filename] = output_data

            if output_path:
                write_output(out_filename, output_path, output_data)

    if generate_exception:
        raise generate_exception

    return result_data



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import re

from .meta import NAME, VERSION, DESCRIPTION

parser = argparse.ArgumentParser(
    prog='%s %s' % (NAME, VERSION),
    description=DESCRIPTION,
    epilog='The Cake is a lie.',
)

# Group Main
group_main = parser.add_argument_group(title='Main options')

group_main.add_argument(
    '-t', '--template',
    help='Configuration template set name (packaged, or user provided path)'
)

group_main.add_argument(
    '-p', '--profile',
    help='Configuration data profile name (packaged, or user provided path)',
)

group_main.add_argument(
    '-o', '--output',
    help='Output path to generated files to'
)

group_main.add_argument(
    '--tune',
    help=(
        'Fine tune profile variables by providing a YAML file'
        ' with values mapping (if selected profile support variables)'
    ),
    action='append',
)

# Group Extra
group_extra = parser.add_argument_group(
    title='Extra Options'
)

group_extra.add_argument(
    '-f', '--filter',
    help='Regular expression enabled output filename filter,'
         ' to generate only selected config files',
    action='append',
)

group_extra.add_argument(
    '--save-effective-profile',
    help=(
        'Write used profile data to output directory,'
        ' output has to be specified'
    ),
    action='store_true'
)

# Group Render
group_render = parser.add_argument_group(title='Render options')

group_render.add_argument(
    '--render-generator-notice',
    help='Control generator notice (header) to be put into output files'
         ' (True/False)'
)

group_render.add_argument(
    '--render-licenses',
    help='Control licenses to be put into output files (True/False)'
)

# Group Query
group_query = parser.add_argument_group(title='Query options')

group_query.add_argument(
    '--list-templates',
    help='Print a list of packaged templates',
    action='store_true'
)

group_query.add_argument(
    '--list-profiles',
    help='Print a list of packaged profiles',
    action='store_true'
)

# Group Creator
group_creator = parser.add_argument_group(
    title='Creator Options',
)

group_creator.add_argument(
    '--new-profile',
    help='Export a local copy of an existing profile for further editing'
         ' (local path), you need to use profile option to select a profile'
)

group_creator.add_argument(
    '--new-profile-static',
    help='Export a local copy of a static version of an existing profile'
         ' for further editing (local path), you need to use profile'
         ' option to select a profile'
)

group_creator.add_argument(
    '--new-template',
    help='Export a local copy of selected template for further editing'
         ' (local path), you need to use template to select a template'
)

group_creator.add_argument(
    '--export-tuning',
    help='Export a local copy of profile available variables in yaml'
         ' for easy creation of tuning file (local path)'
)

# Group Logging
group_logging = parser.add_argument_group(title='Logging options')

group_logging.add_argument(
    '-q', '--quiet',
    help='Keep output to minimum, only requested data (listing) or errors',
    action='store_true'
)

group_logging.add_argument(
    '-v', '--verbose',
    help='Print generation status and user relevant info',
    action='store_true'
)

group_logging.add_argument(
    '-d', '--debug',
    help='Print debugging details',
    action='store_true'
)

# Group Misc
group_misc = parser.add_argument_group(title='Miscellaneous')

group_misc.add_argument(
    '--version',
    help='Display version information',
    action='store_true'
)

REX_BOOL_TRUE = re.compile(r'^(([Tt][Rr][Uu][Ee])|[Yy][Ee][Ss]|(1))$')
REX_BOOL_FALSE = re.compile(r'^(([Ff][Aa][Ll][Ss][Ee])|([Nn][Oo])|(0))$')


def boolize(data):
    """Conver input values from string to bool if possible

    :param data: input data from argparse
    :type data: str
    :return: converted boolean value, or None if not specified
    :rtype: bool or None

    :raises ValueError: if conversion failed
    """
    if data is None:
        return None
    if REX_BOOL_TRUE.match(data):
        return True
    if REX_BOOL_FALSE.match(data):
        return False
    raise ValueError('Cannot convert input option "%s" to bool' % data)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock

from amqcfg.query import list_profiles
from ..files.fakes import fake_profile_path, fake_module_path


def fake_os_walk_profiles_basic(*_):
    prefix_path = fake_profile_path()
    return [
        (
            os.path.join(prefix_path, ''),
            ['a', 'b', 'x', '_not'],
            ['profile.yaml']
        ),
        (
            os.path.join(prefix_path, 'a'),
            ['b'],
            ['profile.yaml', 'something.txt']
        ),
        (
            os.path.join(prefix_path, 'a', 'b'),
            [],
            ['profile.yaml']
        ),
        (
            os.path.join(prefix_path, 'b'),
            [],
            ['my_profile.yaml']
        ),
        (
            os.path.join(prefix_path, 'x'),
            ['_not'],
            ['something.txt']
        ),
        (
            os.path.join(prefix_path, 'x', '_not'),
            [],
            ['not_a_profile.yaml']
        ),
        (
            os.path.join(prefix_path, '_not'),
            [],
            ['also_not_a_profile.yaml']
        ),
    ]


@mock.patch('amqcfg.query.get_module_path',
            side_effect=fake_module_path)
@mock.patch('os.walk', side_effect=fake_os_walk_profiles_basic)
def test_basic(*_):
    expected = [
        'profile.yaml',
        'a/profile.yaml',
        'a/b/profile.yaml',
        'b/my_profile.yaml',
    ]
    result = list_profiles()
    assert expected == result


def fake_os_walk_not_a_profile(*_):
    prefix_path = fake_profile_path()
    return [
        (
            os.path.join(prefix_path, 'x'),
            ['_not'],
            ['something.txt']
        ),
        (
            os.path.join(prefix_path, 'x', '_not'),
            [],
            ['not_a_profile.yaml']
        ),
        (
            os.path.join(prefix_path, '_not'),
            [],
            ['also_not_a_profile.yaml']
        ),
    ]


@mock.patch('amqcfg.query.get_module_path',
            side_effect=fake_module_path)
@mock.patch('os.walk', side_effect=fake_os_walk_not_a_profile)
def test_not_a_profile(*_):
    expected = []
    result = list_profiles()
    assert expected == result


@mock.patch('amqcfg.query.get_module_path',
            side_effect=fake_module_path)
@mock.patch('os.walk', side_effect=((),))
def test_empty(*_):
    expected = []
    result = list_profiles()
    assert expected == result



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import pytest

import amqcfg.profiles
from amqcfg.exceptions import TemplateError
from amqcfg.profiles import get_profile_template
from ..files.fakes import fake_select_profile_file, fake_profile_path


@mock.patch('amqcfg.profiles.select_profile_file',
            side_effect=fake_select_profile_file)
@mock.patch('os.path.isdir', side_effect=(True,))
@mock.patch('amqcfg.profiles.get_profiles_path',
            side_effect=fake_profile_path)
@mock.patch('amqcfg.profiles.FileSystemLoader', mock.Mock())
@mock.patch('amqcfg.profiles.Environment', mock.Mock())
def test_true(*_):
    profile_name = 'profile.yaml'
    expected_template = 'my_template'
    expected_selected_name, expected_selected_path = \
        fake_select_profile_file(profile_name)

    fake_env = mock.Mock()
    amqcfg.profiles.Environment.return_value = fake_env
    fake_env.get_template.return_value = expected_template

    template = get_profile_template(profile_name)

    assert template == expected_template
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.select_profile_file.assert_called_with(profile_name)
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.FileSystemLoader.assert_called_with([
        expected_selected_path,
        fake_profile_path()
    ])
    fake_env.get_template.assert_called_with(expected_selected_name)


@mock.patch('amqcfg.profiles.select_profile_file',
            side_effect=fake_select_profile_file)
@mock.patch('os.path.isdir', side_effect=(False,))
@mock.patch('amqcfg.profiles.get_profiles_path',
            side_effect=fake_profile_path)
@mock.patch('amqcfg.profiles.FileSystemLoader', mock.Mock())
@mock.patch('amqcfg.profiles.Environment', mock.Mock())
def test_bad_profile_exception(*_):
    profile_name = 'bad_profile.yaml'
    expected_template = 'my_template'

    fake_env = mock.Mock()
    amqcfg.profiles.Environment.return_value = fake_env
    fake_env.get_template.return_value = expected_template

    with pytest.raises(TemplateError):
        get_profile_template(profile_name)

    # noinspection PyUnresolvedReferences
    amqcfg.profiles.select_profile_file.assert_called_with(profile_name)
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.FileSystemLoader.assert_not_called()
    fake_env.get_template.assert_not_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import pytest

import amqcfg.profiles
from amqcfg.exceptions import TemplateError
from amqcfg.profiles import load_profile_defaults
from .fakes import (
    fake_load_profile_defaults,
    fake_get_tuned_profile,
)


@mock.patch('amqcfg.profiles.get_profile_template', mock.Mock())
def test_true(*_):
    profile_name = 'profile.yaml'
    expected_data = fake_load_profile_defaults()

    # fake jinja template
    fake_template = mock.Mock()
    amqcfg.profiles.get_profile_template.return_value = fake_template
    fake_template.render.return_value = fake_get_tuned_profile()

    profile_defaults = load_profile_defaults(profile_name)

    assert profile_defaults == expected_data
    # noinspection PyUnresolvedReferences
    amqcfg.profiles.get_profile_template.assert_called_with(profile_name)


@mock.patch('amqcfg.profiles.get_profile_template',
            side_effect=TemplateError)
def test_bad_template(*_):
    profile_name = 'bad_profile.yaml'

    with pytest.raises(TemplateError):
        load_profile_defaults(profile_name)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os


def fake_module_path():
    return '/module/path'


def fake_profile_path():
    return os.path.join(fake_module_path(), 'profiles')


def fake_templates_path():
    return os.path.join(fake_module_path(), 'templates')


def fake_os_abspath(path):
    return os.path.join('/absolute/path/', path)


def fake_select_profile_file(name):
    return name, fake_profile_path()


def fake_select_template_dir(name):
    return os.path.join(fake_templates_path(), name)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil

import mock as mock
import pytest

import amqcfg.output
from amqcfg.output import new_template
from ..files.fakes import fake_templates_path, fake_select_template_dir


@mock.patch('amqcfg.output.select_template_dir',
            side_effect=fake_select_template_dir)
@mock.patch('shutil.copytree', mock.Mock())
def test_true(*_):
    template_name = 'my_template/1.2.3'
    template_path = fake_templates_path()
    template = os.path.join(template_path, template_name)
    destination_path = '/output/template/dir/'
    destination_name = 'new_template'
    destination = os.path.join(destination_path, destination_name)

    new_template(template_name, destination)
    # noinspection PyUnresolvedReferences
    shutil.copytree.assert_called_with(template, destination,
                                       symlinks=False)


@mock.patch('amqcfg.output.select_template_dir',
            side_effect=fake_select_template_dir)
@mock.patch('shutil.copytree',
            side_effect=OSError('[Errno 13] Permission denied: \'path\''))
def test_bad_destination(*_):
    template_name = 'my_template/1.2.3'
    destination = '/bad/destination'

    with pytest.raises(OSError):
        new_template(template_name, destination)
    # noinspection PyUnresolvedReferences
    shutil.copytree.assert_called()


@mock.patch('amqcfg.output.select_template_dir',
            side_effect=fake_select_template_dir)
@mock.patch('amqcfg.output.ensure_output_path', side_effect=mock.Mock())
def test_no_destination(*_):
    template_name = 'my_template/1.2.3'
    destination = ''

    with pytest.raises(OSError):
        new_template(template_name, destination)
    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_not_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jinja2
import mock
import pytest

import amqcfg.exceptions
import amqcfg.templates
from amqcfg.templates import get_template_environment
from ..files.fakes import fake_select_template_dir
from ..files.fakes import fake_templates_path


@mock.patch('amqcfg.templates.get_templates_path',
            side_effect=fake_templates_path)
@mock.patch('amqcfg.templates.select_template_dir',
            side_effect=fake_select_template_dir)
@mock.patch('amqcfg.templates.FileSystemLoader', mock.Mock())
@mock.patch('amqcfg.templates.Environment', mock.Mock())
def test_true(*_):
    template_name = 'template/1.0.0'
    expected_env = 'template environment'
    expected_template_path = fake_templates_path()
    expected_selected_templates = fake_select_template_dir(template_name)

    amqcfg.templates.Environment.return_value = expected_env

    result = get_template_environment(template_name)

    assert expected_env == result

    # noinspection PyUnresolvedReferences
    amqcfg.templates.select_template_dir.assert_called_with(template_name)
    # noinspection PyUnresolvedReferences
    amqcfg.templates.FileSystemLoader.assert_called_with([
        expected_selected_templates, expected_template_path
    ])


@mock.patch('amqcfg.templates.get_templates_path',
            side_effect=fake_templates_path)
@mock.patch('amqcfg.templates.select_template_dir',
            side_effect=fake_select_template_dir)
@mock.patch('amqcfg.templates.FileSystemLoader',
            side_effect=jinja2.TemplateError)
@mock.patch('amqcfg.templates.Environment', mock.Mock())
def test_jinja_loader_exception(*_):
    template_name = 'template/1.0.0'
    expected_env = 'template environment'

    amqcfg.templates.Environment.return_value = expected_env

    # jinja2 also have a TemplateError exception
    with pytest.raises(amqcfg.exceptions.TemplateError):
        get_template_environment(template_name)


@mock.patch('amqcfg.templates.get_templates_path',
            side_effect=fake_templates_path)
@mock.patch('amqcfg.templates.select_template_dir',
            side_effect=fake_select_template_dir)
@mock.patch('amqcfg.templates.FileSystemLoader', mock.Mock())
@mock.patch('amqcfg.templates.Environment', side_effect=jinja2.TemplateError)
def test_jinja_environment_exception(*_):
    template_name = 'template/1.0.0'
    expected_env = 'template environment'

    amqcfg.templates.Environment.return_value = expected_env

    # jinja2 also have a TemplateError exception
    with pytest.raises(amqcfg.exceptions.TemplateError):
        get_template_environment(template_name)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock
import pytest

import amqcfg.output

from amqcfg.output import write_output


@mock.patch('amqcfg.output.open', mock.MagicMock())
def test_true(*_):
    file_name = 'my_output.txt'
    file_path = '/output/path'
    file_pathname = os.path.join(file_path, file_name)

    data = 'output data'

    write_output(file_name, file_path, data)
    # noinspection PyUnresolvedReferences
    amqcfg.output.open.assert_called_with(file_pathname, 'w')
    fh = amqcfg.output.open.return_value.__enter__.return_value
    fh.write.assert_has_calls([mock.call(data), mock.call(os.linesep)])


@mock.patch('amqcfg.output.open', side_effect=IOError)
def test_bad_filename(*_):
    file_name = 'my_output.txt'
    file_path = '/bad/output/path'
    file_pathname = os.path.join(file_path, file_name)

    data = 'output data'

    with pytest.raises(IOError):
        write_output(file_name, file_path, data)
    # noinspection PyUnresolvedReferences
    amqcfg.output.open.assert_called_with(file_pathname, 'w')



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jinja2
import mock
import pytest

import amqcfg.amqcfg
from amqcfg.amqcfg import generate_outputs
from amqcfg.exceptions import GenerationError


@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
def test_one_true_no_output(*_):
    config_data = {}
    template_list = [
        'broker.xml.jinja2',
    ]

    expected_result_data = {
        'broker.xml': 'Rendered data'
    }

    env = mock.Mock()
    template = mock.Mock()
    env.get_template.return_value = template
    template.render.return_value = 'Rendered data'

    # noinspection PyTypeChecker
    result = generate_outputs(
        config_data=config_data,
        template_list=template_list,
        env=env,
        output_path=None
    )

    assert expected_result_data == result

    env.get_template.assert_called_with('broker.xml.jinja2')
    template.render.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()


@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
def test_one_true_output(*_):
    config_data = {}
    output_file = 'broker.xml'
    output_path = '/output/directory'
    expected_output_data = 'Rendered data'
    template_list = [
        '%s.jinja2' % output_file,
    ]
    expected_result_data = {
        output_file: 'Rendered data'
    }

    env = mock.Mock()
    template = mock.Mock()
    env.get_template.return_value = template
    template.render.return_value = expected_output_data

    # noinspection PyTypeChecker
    result = generate_outputs(
        config_data=config_data,
        template_list=template_list,
        env=env,
        output_path=output_path
    )

    assert expected_result_data == result

    env.get_template.assert_called_with('broker.xml.jinja2')
    template.render.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_called_with(output_file, output_path,
                                                  expected_output_data)


@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
def test_one_exception_render(*_):
    config_data = {}
    output_file = 'broker.xml'
    output_path = '/output/directory'
    template_list = [
        '%s.jinja2' % output_file,
    ]

    env = mock.Mock()
    template = mock.Mock()
    env.get_template.return_value = template
    template.render.side_effect = jinja2.TemplateError

    with pytest.raises(GenerationError):
        # noinspection PyTypeChecker
        generate_outputs(
            config_data=config_data,
            template_list=template_list,
            env=env,
            output_path=output_path
        )

    env.get_template.assert_called_with('broker.xml.jinja2')
    template.render.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()


@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
def test_one_exception_template(*_):
    config_data = {}
    output_file = 'broker.xml'
    output_path = '/output/directory'
    template_list = [
        '%s.jinja2' % output_file,
    ]

    env = mock.Mock()
    env.get_template.side_effect = jinja2.TemplateError

    with pytest.raises(GenerationError):
        # noinspection PyTypeChecker
        generate_outputs(
            config_data=config_data,
            template_list=template_list,
            env=env,
            output_path=output_path
        )

    env.get_template.assert_called_with('broker.xml.jinja2')
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from amqcfg.query import get_main_template_list


def fake_list_templates(filter_func):
    file_list = [
        'broker.xml.jinja2',
        'artemis-users.properties.jinja2',
        'not_a_template.xml',
    ]
    return [i for i in file_list if filter_func(i)]


def test_basic(*_):
    expected = [
        'broker.xml.jinja2',
        'artemis-users.properties.jinja2',
    ]
    env = mock.Mock()
    env.list_templates = fake_list_templates

    result = get_main_template_list(env)
    assert expected == result



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import shutil

import yaml

from .exceptions import ProfileError
from .files import select_profile_file, ensure_output_path, select_template_dir
from .meta import NAME
from .profiles import load_tuned_profile, load_profile_defaults

LOG = logging.getLogger(__name__)


def new_profile(profile, dest_profile):
    """Export an existing profile

    :param profile: existing profile name (from user or packaged)
    :type profile: str
    :param dest_profile: filename of new profile
    :type dest_profile: str

    :raises OSError: if there is a problem with destination path
    :raises ProfileError: if there is a problem with the existing template
    """
    profile_name, profile_path = select_profile_file(profile)
    src = os.path.join(profile_path, profile_name)

    dest_path = os.path.dirname(dest_profile)
    if dest_path:
        ensure_output_path(dest_path)  # raises: OSError

    shutil.copyfile(src, dest_profile)


def new_profile_rendered(profile, dest_profile, tuning_files=None):
    """Export an existing profile in static form, stripped defaults section

    :param profile: existing profile name (from user or packaged)
    :type profile: str
    :param dest_profile: filename of new profile
    :type dest_profile: str
    :param tuning_files: user specified tuning file path
    :type tuning_files: list[str] or None

    :raises OSError: if there is a problem with destination path
    :raises ProfileError: if there is a problem with the existing template
    """
    config_data, _ = load_tuned_profile(profile, tuning_files)

    dest_path = os.path.dirname(dest_profile)
    dest_name = os.path.basename(dest_profile)
    if dest_path:
        ensure_output_path(dest_path)  # raises: OSError

    if '_defaults' in config_data:
        del config_data['_defaults']

    export_data = yaml.dump(config_data, default_flow_style=False)
    export_data = '# %s tuning file generated from profile %s%s%s' % (
        NAME, profile, os.linesep, export_data
    )
    write_output(dest_name, dest_path, export_data)


def new_template(template, dest_template):
    """Export an existing template dir

    :param template: existing template name (from user or packaged)
    :type template: str
    :param dest_template: dirname of new profile
    :type dest_template: str

    :raises OSError: if there is problem with destination path
    :raises TemplateError: if there is a problem with existing template
    """
    template_path = select_template_dir(template)

    shutil.copytree(template_path, dest_template, symlinks=False)


def export_tuning_variables(profile_name, dest_file):
    """Export subsection of profile tunable variables to new yaml file

    :param profile_name: profile name (user defined or packaged)
    :type profile_name: str
    :param dest_file: filename of destination tuning file
    :type dest_file: str

    :raises OSError: if there is s problem with destination path
    :raises ProfileError: if there is a problem with selected profile
    """

    defaults_data = load_profile_defaults(profile_name)
    if not defaults_data:
        raise ProfileError(
            'Selected profile "%s" does not contain any tunable variables'
            % profile_name
        )

    dest_path = os.path.dirname(dest_file)
    dest_name = os.path.basename(dest_file)
    if dest_path:
        ensure_output_path(dest_path)  # raises: OSError

    export_data = yaml.dump(defaults_data, default_flow_style=False)
    LOG.debug('Exported tuning data:\n%s', export_data)
    export_data = '# %s tuning file generated from profile %s%s%s' % (
        NAME, profile_name, os.linesep, export_data
    )
    write_output(dest_name, dest_path, export_data)
    LOG.info('Tuning data exported')


def write_output(filename, output_path, data):
    """Helper to write generated data to file

    :param filename: name of output file
    :type filename: str
    :param output_path: main path for generating files
    :type output_path: str
    :param data: data to write
    :type data: str
    """
    filepath = os.path.join(output_path, filename)
    with open(filepath, 'w') as fh:
        fh.write(data)
        fh.write(os.linesep)
    LOG.debug('File writen "%s"', filepath)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import mock
import shutil

import pytest

from amqcfg.output import new_profile
import amqcfg.output
from ..files.fakes import (
    fake_select_profile_file,
    fake_profile_path
)


@mock.patch('amqcfg.output.select_profile_file',
            side_effect=fake_select_profile_file)
@mock.patch('shutil.copyfile', mock.Mock())
@mock.patch('amqcfg.output.ensure_output_path', mock.Mock())
def test_true(*_):
    """Creating new profile true path"""
    profile_name = 'existing_profile.yaml'
    destination = '/output/directory/my_new_profile.yaml'
    expected_source_file = os.path.join(fake_profile_path(), profile_name)
    expected_target_file = destination
    expected_target_dir = os.path.dirname(destination)

    new_profile(profile_name, destination)
    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_called_with(expected_target_dir)
    # noinspection PyUnresolvedReferences
    shutil.copyfile.assert_called_with(expected_source_file,
                                       expected_target_file)


@mock.patch('amqcfg.output.select_profile_file',
            side_effect=fake_select_profile_file)
@mock.patch('shutil.copyfile', mock.Mock())
@mock.patch('amqcfg.output.ensure_output_path', mock.Mock())
def test_true_no_destination(*_):
    """Creating new profile true path"""
    profile_name = 'existing_profile.yaml'
    destination = 'my_new_profile.yaml'
    expected_source_file = os.path.join(fake_profile_path(), profile_name)
    expected_target_file = destination

    new_profile(profile_name, destination)
    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    shutil.copyfile.assert_called_with(expected_source_file,
                                       expected_target_file)


@mock.patch('amqcfg.output.select_profile_file',
            side_effect=fake_select_profile_file)
@mock.patch('shutil.copyfile', mock.Mock())
@mock.patch('amqcfg.output.ensure_output_path',
            side_effect=OSError('[Errno 13] Permission denied: \'path\''))
def test_destination_problem_exception(*_):
    """Creating new profile true path"""
    profile_name = 'existing_profile.yaml'
    destination = '/problematic/directory/my_new_profile.yaml'
    expected_target_dir = os.path.dirname(destination)

    with pytest.raises(OSError):
        new_profile(profile_name, destination)
    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_called_with(expected_target_dir)
    # noinspection PyUnresolvedReferences
    shutil.copyfile.assert_not_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
import amqcfg.config_data

dataset_render_config = (
    (amqcfg.config_data.RenderOptions(None, None), 'generator_notice', None),
    (amqcfg.config_data.RenderOptions(None, None), 'licenses', None),
    (amqcfg.config_data.RenderOptions(True, None), 'generator_notice', True),
    (amqcfg.config_data.RenderOptions(False, None), 'generator_notice', False),
    (amqcfg.config_data.RenderOptions(None, True), 'licenses', True),
    (amqcfg.config_data.RenderOptions(None, False), 'licenses', False),
)


@pytest.mark.parametrize('render_options,key,value', dataset_render_config)
def test_add_render_config_members(render_options, key, value):
    data = {}
    amqcfg.config_data.add_render_config(config_data=data,
                                         render_options=render_options)
    if value is not None:
        assert key in data['render']
    else:
        assert key not in data['render']


@pytest.mark.parametrize('render_options,key,value', dataset_render_config)
def test_add_render_config_values(render_options, key, value):
    data = {}
    amqcfg.config_data.add_render_config(config_data=data,
                                         render_options=render_options)
    if value is not None:
        assert data['render'][key] == value
    else:
        assert True



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os

import yaml
from jinja2 import Environment, FileSystemLoader

from .exceptions import ProfileError, TemplateError
from .files import select_profile_file, get_profiles_path

LOG = logging.getLogger(__name__)


def load_tuned_profile(profile, tuning_files):
    """Load a complete tuned profile, get config_data as well
    as rendered profile

    :param profile: profile name (packaged or user specified)
    :type profile: str
    :param tuning_files: tuning file to tune variables
    :type tuning_files: list[str] or None

    :raises ProfileError: if there is problem with loading the profile data
    :raises ProfileError: when there is problem opening tuning file
    :raises ProfileError: when there is problem with tuning file parsing
    :raises ProfileError: when there is problem with loading profile

    :return: config_data and rendered tuned profile
    :rtype: tuple[dict, str]
    """
    # fine tuning profile
    tuned_profile = get_tuned_profile(profile, tuning_files)
    # loading tuned profile data
    try:
        config_data = yaml.load(tuned_profile)
    except yaml.YAMLError as exc:
        raise ProfileError(
            'Unable to parse YAML profile "%s" tuned by "%s" %s'
            % (profile, tuning_files, exc))
    return config_data, tuned_profile


def get_tuned_profile(profile, tuning_files):
    """Get selected profile and use tuning file to fine tune
    it's variable values.

    :param profile: profile name (packaged) or path to profile file (user)
    :type profile: str
    :param tuning_files: path to user specified tuning file (yaml)
    :type tuning_files: list[str] or None

    :raises ProfileError: when there is problem opening tuning file
    :raises ProfileError: when there is problem with tuning file parsing
    :raises ProfileError: when there is problem with loading profile

    :return: fine tuned profile yaml to be loaded for templating
    :rtype: str
    """
    tuning_data = load_profile_defaults(profile)

    if tuning_files:
        for tuning_file in tuning_files:
            try:
                tuning_values = yaml.load(open(tuning_file, 'r'))
            except IOError as exc:
                raise ProfileError(
                    'Unable to open tuning file "%s" %s' % (tuning_file, exc)
                )
            except yaml.YAMLError as exc:
                raise ProfileError(
                    'Unable to parse YAML tuning file "%s" %s'
                    % (tuning_files, exc)
                )
            tuning_data.update(tuning_values)
            LOG.debug('Profile "%s" tuned with "%s"', profile, tuning_file)
    else:
        LOG.debug('Profile "%s" using _defaults', profile)

    tuning_profile = get_profile_template(profile)
    tuned_profile = tuning_profile.render(tuning_data)

    return tuned_profile


def load_profile_defaults(profile):
    """Load defaults variables from a profile if available

    .. note: profile will be rendered as scratch without any values
        to be able to be loaded as valid yaml.

    :param profile: profile name (from package or from user)
    :type profile: str

    :return: defaults values mapping, if not available then empty dict
    :rtype: dict
    """
    # scratch render of profile template for _defaults extraction
    scratch_profile = get_profile_template(profile)
    scratch_profile = scratch_profile.render()
    tmp_data = yaml.load(scratch_profile)
    tuning_data = tmp_data.get('_defaults', {})
    LOG.debug('Tuning data: %s', tuning_data)
    return tuning_data


def get_profile_template(profile_name):
    """Get a jinja2 template via env generated for selected profile
    (for fine-tuning of profile)

    :param profile_name: name of template set
        (alternatively path to user specified template set)
    :type profile_name: str

    :return: jinja2 profile template for fine tuning template
    :rtype: Template
    """

    selected_template_name, selected_template_path = \
        select_profile_file(profile_name)

    if not os.path.isdir(selected_template_path):
        raise TemplateError(
            'Unable to load requested profile location "%s"' % profile_name
        )

    env = Environment(
        loader=FileSystemLoader([
            selected_template_path,  # selected template
            get_profiles_path(),
        ]),
        trim_blocks=True,
        lstrip_blocks=True,
    )

    template = env.get_template(selected_template_name)

    return template



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock
import pytest

import amqcfg.files
from amqcfg.exceptions import TemplateError
from .fakes import fake_templates_path


@mock.patch('amqcfg.files.get_templates_path', side_effect=fake_templates_path)
@mock.patch('os.path.isdir', side_effect=(False, True))
@mock.patch('os.path.isfile', side_effect=(True,))
def test_packaged_true(*_):
    """Packaged template selection"""
    template_name = 'product/1.0.0'
    expected_dir = os.path.join(fake_templates_path(), template_name)
    assert amqcfg.files.select_template_dir(template_name) == expected_dir


@mock.patch('amqcfg.files.get_templates_path', side_effect=fake_templates_path)
@mock.patch('os.path.isdir', side_effect=(True, True))
@mock.patch('os.path.isfile', side_effect=(True,))
def test_user_true(*_):
    """User specified template selection"""
    template_name = 'user_product/1.0.0'
    assert amqcfg.files.select_template_dir(template_name) == template_name


@mock.patch('amqcfg.files.get_templates_path', side_effect=fake_templates_path)
@mock.patch('os.path.isdir', side_effect=(False, False))
@mock.patch('os.path.isfile', side_effect=(True,))
def test_not_directory(*_):
    """Template set not a directory"""
    template_name = 'user_product/1.0.0'
    with pytest.raises(TemplateError):
        assert amqcfg.files.select_template_dir(template_name) == template_name


@mock.patch('amqcfg.files.get_templates_path', side_effect=fake_templates_path)
@mock.patch('os.path.isdir', side_effect=(False, True))
@mock.patch('os.path.isfile', side_effect=(False,))
def test_packaged_missing_template_marker(*_):
    """Packaged Template set not a directory"""
    template_name = 'user_product/1.0.0'
    with pytest.raises(TemplateError):
        assert amqcfg.files.select_template_dir(template_name) == template_name


@mock.patch('amqcfg.files.get_templates_path', side_effect=fake_templates_path)
@mock.patch('os.path.isdir', side_effect=(True, True))
@mock.patch('os.path.isfile', side_effect=(False,))
def test_user_missing_template_marker(*_):
    """User specified Template set not a directory"""
    template_name = 'user_product/1.0.0'
    with pytest.raises(TemplateError):
        assert amqcfg.files.select_template_dir(template_name) == template_name



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import pytest

from amqcfg.exceptions import ProfileError
from amqcfg.profiles import load_tuned_profile
from ..profiles.fakes import (
    fake_load_tuned_profile_with_defaults,
    fake_get_tuned_profile,
)


@mock.patch('amqcfg.profiles.get_tuned_profile',
            side_effect=fake_get_tuned_profile)
def test_true(*_):
    profile_name = 'my_profile.yaml'
    expected_data, expected_profile = fake_load_tuned_profile_with_defaults()

    config_data, tuned_profile = load_tuned_profile(profile_name, None)

    assert config_data == expected_data
    assert tuned_profile == expected_profile


@mock.patch('amqcfg.profiles.get_tuned_profile',
            side_effect=('%this is not yaml',))
def test_bad_data_exception(*_):
    profile_name = 'my_profile.yaml'

    with pytest.raises(ProfileError):
        load_tuned_profile(profile_name, None)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock
import pytest

import amqcfg.output
from amqcfg.exceptions import ProfileError
from amqcfg.meta import NAME
from amqcfg.output import export_tuning_variables
from ..profiles.fakes import fake_load_profile_defaults, \
    fake_profile_defaults_yaml


@mock.patch('amqcfg.output.load_profile_defaults',
            side_effect=fake_load_profile_defaults)
@mock.patch('amqcfg.output.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.output.write_output', mock.Mock())
def test_true(*_):
    profile_name = 'my_profile.yaml'
    destination_path = '/destination/path/tuning'
    destination_name = 'tuning.yaml'
    destination = os.path.join(destination_path, destination_name)

    expected_data = fake_profile_defaults_yaml()
    expected_data = '# %s tuning file generated from profile %s%s%s' % (
        NAME, profile_name, os.linesep, expected_data
    )

    export_tuning_variables(profile_name, destination)

    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_called_with(destination_path)
    # noinspection PyUnresolvedReferences
    amqcfg.output.write_output.assert_called_with(destination_name,
                                                  destination_path,
                                                  expected_data)


@mock.patch('amqcfg.output.load_profile_defaults',
            side_effect=({},))
@mock.patch('amqcfg.output.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.output.write_output', mock.Mock())
def test_no_defaults_exception(*_):
    profile_name = 'my_profile.yaml'
    destination_path = '/destination/path/tuning'
    destination_name = 'tuning.yaml'
    destination = os.path.join(destination_path, destination_name)

    with pytest.raises(ProfileError):
        export_tuning_variables(profile_name, destination)

    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.output.write_output.assert_not_called()


@mock.patch('amqcfg.output.load_profile_defaults',
            side_effect=fake_load_profile_defaults)
@mock.patch('amqcfg.output.ensure_output_path',
            side_effect=OSError('[Errno 13] Permission denied: \'path\''))
@mock.patch('amqcfg.output.write_output', mock.Mock())
def test_bad_destination_exception(*_):
    profile_name = 'my_profile.yaml'
    destination_path = '/bad/destination'
    destination_name = 'tuning.yaml'
    destination = os.path.join(destination_path, destination_name)

    with pytest.raises(OSError):
        export_tuning_variables(profile_name, destination)

    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_called_with(destination_path)
    # noinspection PyUnresolvedReferences
    amqcfg.output.write_output.assert_not_called()


@mock.patch('amqcfg.output.load_profile_defaults',
            side_effect=fake_load_profile_defaults)
@mock.patch('amqcfg.output.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.output.write_output',
            side_effect=OSError('[Errno 13] Permission denied: \'path\''))
def test_no_destination_exception(*_):
    profile_name = 'my_profile.yaml'
    destination_path = ''
    destination_name = 'tuning.yaml'
    destination = os.path.join(destination_path, destination_name)

    with pytest.raises(OSError):
        export_tuning_variables(profile_name, destination)

    # noinspection PyUnresolvedReferences
    amqcfg.output.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.output.write_output.assert_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import time
from collections import namedtuple

from .meta import NAME


def add_template_metadata(config_data):
    """add templating metadata to original template data

    :param config_data:  original template data
    :type config_data: dict
    """
    now = datetime.datetime.now()

    config_data['metadata'] = {
        'tool_name': NAME,
        'datetime': {
            'datetime': now.strftime('%Y-%m-%d %H:%M'),
            'year': now.strftime('%Y'),
            'time': now.strftime('%H:%M'),
            'date': now.strftime('%Y-%m-%d'),
            'unix': time.time(),
        }
    }


def add_render_config(config_data, render_options):
    """add render related config to original template data

    :param config_data: original template data
    :type config_data: dict
    :param render_options: render options to tune up rendering
    :type render_options: RenderOptions
    """
    render_config = {}

    if render_options.generator_notice is not None:
        render_config['generator_notice'] = render_options.generator_notice
    if render_options.licenses is not None:
        render_config['licenses'] = render_options.licenses

    # update loaded config data
    if 'render' not in config_data:
        config_data['render'] = render_config
    else:
        config_data['render'].update(render_config)


RenderOptions = namedtuple('RenderOptions', ['generator_notice', 'licenses'])



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def fake_get_tuned_profile_data(*_):
    return {'_defaults': {'default_key': 'default_value'}, 'key': 'value'}


def fake_get_tuned_profile(*_):
    return (
        '_defaults:\n'
        '  default_key: default_value\n'
        ''
        'key: value\n'
    )


def fake_load_tuned_profile_with_defaults(*_):
    return (
        fake_get_tuned_profile_data(),
        fake_get_tuned_profile()
    )


def fake_load_tuned_profile_no_defaults(*_):
    return (
        {'key': 'value'},
        (
            'key: value\n'
        )
    )


def fake_load_tuned_profile_w_template(*_):
    return (
        {'render': {'template': 'template/1.0.0'}, 'key': 'value'},
        (
            '_render:\n'
            '  template: template/1.0.0\n'
            '\n'
            'key: value\n'
        )
    )


def fake_load_profile_defaults(*_):
    return {'default_key': 'default_value'}


def fake_profile_defaults_yaml(*_):
    return (
        'default_key: default_value\n'
    )



#!/usr/bin/env python

# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup

# Version reading from amqcfg/meta.py
# according to (3b) from:
# https://packaging.python.org/guides/single-sourcing-package-version/
meta = {}
with open('./amqcfg/meta.py', 'r') as meta_file:
    exec (meta_file.read(), meta)

setup(
    name=meta.get('NAME'),
    version=meta.get('VERSION'),
    packages=['amqcfg'],
    entry_points={
        'console_scripts': [
            'amqcfg = amqcfg.amqcfg_cli:main',
        ],
    },
    install_requires=[
        'jinja2',
        'pyyaml',
    ],
    setup_requires=[
        'pytest-runner'
    ],
    tests_require=[
        'pytest',
        'mock',
    ],
    url='https://bitbucket.org/msgqe/amqcfg',
    license='Apache-2.0',
    author='Zdenek Kraus',
    author_email='zkraus@redhat.com',
    description=meta.get('SHORT_DESCRIPTION'),
    long_description=meta.get('DESCRIPTION'),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Code Generators',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing',
        'Topic :: System :: Installation/Setup',
        'Topic :: System :: Systems Administration',
        'Topic :: Text Processing',
        'Topic :: Utilities'
    ],
)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
import amqcfg.amqcfg
from amqcfg.amqcfg import generate
from amqcfg.exceptions import ProfileError, TemplateError

from ..profiles.fakes import (
    fake_load_tuned_profile_no_defaults,
    fake_load_tuned_profile_w_template
)

import mock


@mock.patch('amqcfg.amqcfg.load_tuned_profile',
            side_effect=fake_load_tuned_profile_no_defaults)
@mock.patch('amqcfg.amqcfg.add_template_metadata', mock.Mock())
@mock.patch('amqcfg.amqcfg.add_render_config', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_true_render_options(*_):
    profile = 'profile.yaml'
    template = 'template/1.0.0'
    render_options = 'Render options'
    expected_result = 'generated data'

    config_data, _ = fake_load_tuned_profile_no_defaults()

    amqcfg.amqcfg.generate_outputs.return_value = expected_result

    result = generate(
        profile=profile,
        template=template,
        render_options=render_options
    )

    assert expected_result == result

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.add_render_config.assert_called_with(config_data,
                                                       render_options)
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_true_tuning_files(*_):
    profile = 'profile.yaml'
    template = 'template/1.0.0'
    tuning_files = ['tune1.yaml', 'tune2.yaml']
    expected_result = 'generated data'

    amqcfg.amqcfg.load_tuned_profile.side_effect = \
        fake_load_tuned_profile_no_defaults
    amqcfg.amqcfg.generate_outputs.return_value = expected_result

    result = generate(
        profile=profile,
        template=template,
        tuning_files=tuning_files,
    )

    assert expected_result == result

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.load_tuned_profile.assert_called_with(profile, tuning_files)
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile',
            side_effect=fake_load_tuned_profile_no_defaults)
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_true_no_output_path_write_profile(*_):
    profile = 'profile.yaml'
    template = 'template/1.0.0'
    expected_result = 'generated data'

    amqcfg.amqcfg.generate_outputs.return_value = expected_result

    result = generate(
        profile=profile,
        template=template,
        write_profile_data=True,
    )

    assert expected_result == result

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile',
            side_effect=fake_load_tuned_profile_no_defaults)
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_true_output_path_write_profile(*_):
    profile = 'profile.yaml'
    template = 'template/1.0.0'
    output_path = '/out/directory'
    expected_result = 'generated data'

    amqcfg.amqcfg.generate_outputs.return_value = expected_result

    result = generate(
        profile=profile,
        template=template,
        output_path=output_path,
        write_profile_data=True,
    )

    assert expected_result == result

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile',
            side_effect=fake_load_tuned_profile_no_defaults)
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_true_output_path(*_):
    profile = 'profile.yaml'
    template = 'template/1.0.0'
    output_path = '/out/directory'
    expected_result = 'generated data'

    amqcfg.amqcfg.generate_outputs.return_value = expected_result

    result = generate(
        profile=profile,
        template=template,
        output_path=output_path,
    )

    assert expected_result == result

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile',
            side_effect=fake_load_tuned_profile_no_defaults)
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_true_template(*_):
    profile = 'profile.yaml'
    template = 'template/1.0.0'
    expected_result = 'generated data'

    amqcfg.amqcfg.generate_outputs.return_value = expected_result

    result = generate(
        profile=profile,
        template=template,
    )

    assert expected_result == result

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile',
            side_effect=fake_load_tuned_profile_w_template)
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_true_profile_template(*_):
    profile = 'profile.yaml'
    expected_result = 'generated data'

    amqcfg.amqcfg.generate_outputs.return_value = expected_result

    result = generate(profile=profile)

    assert expected_result == result

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile',
            side_effect=fake_load_tuned_profile_no_defaults)
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_no_template_exception(*_):
    profile = 'profile.yaml'

    with pytest.raises(TemplateError):
        generate(profile=profile)

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_not_called()


@mock.patch('amqcfg.amqcfg.load_tuned_profile', side_effect=ProfileError)
@mock.patch('amqcfg.amqcfg.get_template_environment', mock.Mock())
@mock.patch('amqcfg.amqcfg.get_main_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.filter_template_list', mock.Mock())
@mock.patch('amqcfg.amqcfg.ensure_output_path', mock.Mock())
@mock.patch('amqcfg.amqcfg.write_output', mock.Mock())
@mock.patch('amqcfg.amqcfg.generate_outputs', mock.Mock())
def test_bad_profile_exception(*_):
    profile = 'bad_profile.yaml'

    with pytest.raises(ProfileError):
        generate(profile=profile)

    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_template_environment.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.get_main_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.filter_template_list.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.ensure_output_path.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.write_output.assert_not_called()
    # noinspection PyUnresolvedReferences
    amqcfg.amqcfg.generate_outputs.assert_not_called()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
import amqcfg.cli_arguments

dataset_boolize = (
    (None, None),
    ('Yes', True),
    ('No', False),
    ('yes', True),
    ('no', False),
    ('YES', True),
    ('NO', False),
    ('True', True),
    ('False', False),
    ('true', True),
    ('false', False),
    ('TRUE', True),
    ('FALSE', False),
    ('1', True),
    ('0', False),
)


@pytest.mark.parametrize('value,result', dataset_boolize)
def test_boolize_values(value, result):
    assert amqcfg.cli_arguments.boolize(value) == result


def test_boolize_exception():
    with pytest.raises(ValueError):
        amqcfg.cli_arguments.boolize('test')
